use askama::Template;
use axum::extract::{Path, Query, State};
use axum::Form;
use axum::http::{HeaderMap, StatusCode};
use axum::response::Html;
use serde::Deserialize;

use crate::{err::Error, form, model::state::AppState, Result, view};
use crate::service::MemberService;
use crate::util::redirect;

#[derive(Deserialize)]
pub struct PageQuery {
    pub page: Option<u32>,
    pub msg: Option<String>,
}


pub async fn index(
    State(state): State<AppState>,
    Query(q): Query<PageQuery>,
) -> Result<Html<String>> {
    let tpl = MemberService::index(&state, q).await?;
    let html = tpl.render().map_err(Error::from)?;
    Ok(Html(html))
}

pub async fn detail(
    State(state): State<AppState>,
    Path(id): Path<u32>,
) -> Result<Html<String>> {
    let tpl = MemberService::detail(&state, id).await?;
    let html = tpl.render().map_err(Error::from)?;
    Ok(Html(html))
}


pub async fn add_ui() -> Result<Html<String>> {
    let tpl = view::Add;
    let html = tpl.render().map_err(Error::from)?;
    Ok(Html(html))
}


pub async fn add(
    State(state): State<AppState>,
    Form(frm): Form<form::AddAndEdit>,
) -> Result<(StatusCode, HeaderMap, ())> {
    MemberService::add(&state, frm).await?;
    redirect("/?msg=会员添加成功")
}


pub async fn edit_ui(
    State(state): State<AppState>,
    Path(id): Path<u32>,
) -> Result<Html<String>> {
    let tpl = MemberService::edit_ui(&state, id).await?;
    let html = tpl.render().map_err(Error::from)?;
    Ok(Html(html))
}


pub async fn edit(
    State(state): State<AppState>,
    Path(id): Path<u32>,
    Form(frm): Form<form::AddAndEdit>,
) -> Result<(StatusCode, HeaderMap, ())> {
    MemberService::edit(&state, id, frm).await?;
    redirect("/?msg=会员修改成功")
}

pub async fn del(
    State(state): State<AppState>,
    Path(id): Path<u32>,
) -> Result<(StatusCode, HeaderMap, ())> {
    MemberService::del(&state, id).await?;
    redirect("/?msg=逻辑删除成功")
}

pub async fn real_del(
    State(state): State<AppState>,
    Path(id): Path<u32>,
) -> Result<(StatusCode, HeaderMap, ())> {
    MemberService::real_del(&state, id).await?;
    redirect("/?msg=物理删除成功")
}


pub async fn tran_ui() -> Result<Html<String>> {
    let tpl = view::Tran;
    let html = tpl.render().map_err(Error::from)?;
    Ok(Html(html))
}


pub async fn tran(
    State(state): State<AppState>,
    Form(frm): Form<form::Tran>,
) -> Result<(StatusCode, HeaderMap, ())> {
    let aff = MemberService::tran(&state, frm).await?;
    tracing::debug!("{:?}", aff);
    redirect("/?msg=转账成功")
}
