use crate::{form, model, view, util, Result};
use crate::db::member;
use crate::err::Error;
use crate::handler::PageQuery;
use crate::model::state::AppState;

pub struct MemberService;

impl MemberService {
    pub async fn index(state: &AppState, q: PageQuery) -> Result<view::Home> {
        let conn = util::get_conn(&state);

        let p = member::list(&conn, q.page.unwrap_or(0)).await?;

        return Ok(view::Home { p, msg: q.msg });
    }

    pub async fn detail(state: &AppState, id: u32) -> Result<view::Detail> {
        let conn = util::get_conn(&state);

        let m = member::find(&conn, id).await?;

        match m {
            None => Err(Error::not_found("不存在的会员")),
            Some(m) => Ok(view::Detail { m })
        }
    }

    pub async fn add(state: &AppState, frm: form::AddAndEdit) -> Result<u32> {
        let conn = util::get_conn(&state);

        member::add(&conn, &model::member::Member {
            name: frm.name,
            balance: frm.balance,
            types: frm.types,
            dateline: chrono::Local::now(),
            ..Default::default()
        }).await
    }

    pub async fn edit(state: &AppState, id: u32, frm: form::AddAndEdit) -> Result<u64> {
        let conn = util::get_conn(&state);

        member::edit(&conn, &model::member::Member {
            id,
            name: frm.name,
            balance: frm.balance,
            types: frm.types,
            ..Default::default()
        }).await
    }

    pub async fn edit_ui(state: &AppState, id: u32) -> Result<view::Edit> {
        let conn = util::get_conn(&state);
        let m = member::find(&conn, id).await?;

        match m {
            None => Err(Error::not_found("不存在的会员")),
            Some(m) => Ok(view::Edit { m })
        }
    }


    pub async fn del(state: &AppState, id: u32) -> Result<u64> {
        let conn = util::get_conn(&state);
        member::del(&conn, id).await
    }

    pub async fn real_del(state: &AppState, id: u32) -> Result<u64> {
        let conn = util::get_conn(&state);
        member::real_del(&conn, id).await
    }

    pub async fn tran(state: &AppState, frm: form::Tran) -> Result<(u64, u64)> {
        let conn = util::get_conn(&state);

        member::tran(&conn, &model::member::Tran {
            from_member: frm.from_member,
            to_member: frm.to_member,
            amount: frm.amount,
        }).await
    }
}