use std::sync::Arc;

use axum::http::{header, HeaderMap, StatusCode};

use crate::model::state::AppState;

pub fn get_conn(state: &AppState) -> Arc<sqlx::MySqlPool> {
    state.pool.clone()
}

pub fn redirect(url: &str) -> crate::Result<(StatusCode, HeaderMap, ())> {
    let mut header = HeaderMap::new();
    header.insert(header::LOCATION, url.parse().unwrap());

    Ok((StatusCode::FOUND, header, ()))
}


pub mod default_datetime_format {
    use chrono::{DateTime, Local, NaiveDateTime};
    use serde::{self, Deserialize, Deserializer, Serializer};

    const FORMAT: &'static str = "%Y-%m-%d %H:%M:%S";

    pub fn serialize<S>(
        date: &DateTime<Local>,
        serializer: S,
    ) -> Result<S::Ok, S::Error>
        where
            S: Serializer,
    {
        let s = format!("{}", date.format(FORMAT));
        serializer.serialize_str(&s)
    }

    pub fn deserialize<'de, D>(
        deserializer: D,
    ) -> Result<DateTime<Local>, D::Error>
        where
            D: Deserializer<'de>,
    {
        let datetime_str = String::deserialize(deserializer)?;
        let dt = NaiveDateTime::parse_from_str(&datetime_str, FORMAT).map_err(serde::de::Error::custom)?;
        Ok(dt.and_local_timezone(Local).unwrap())
    }
}